/**
 * 创建人:  @author 北风
 * 创建时间:  2023年05月31日 13:32
 * 项目名称:  YSpeaManager
 * 文件名称:  UtilDao
 * 文件描述:  @Description: 工具类dao
 */
package com.bonus.hn.bearing.capacity.backstage.manager.dao;

import com.bonus.hn.bearing.capacity.backstage.manager.model.SysUser;
import org.apache.ibatis.annotations.Mapper;

@Mapper
public interface UtilDao {
    /**
     * 判断某个数据是否存在
     * 需要传入的值:
     *  tableName:  表名
     *  filed: 字段名
     *  filedValue: 字段值
     * id : 数据库表的id   可加可不加
     * @return 0 表示不存在  1表示存在   sql限制了查询到一条即返回
     */
    int determineIfExists(String tableName,String filedId,String filedValue,String id);


    int personIfExists(String tableName, String filedId, String filedValue);

//    void insertLoginInfo(SysUser b);

    int insertLoginInfo(SysUser b);

    int phoneIfExists(String sysUser, String phone, String phone1, String id);

    int upLoginLog(SysUser bean);
//
//    /**
//     * 获取医院下拉框 可根据传入mapBean value值进行模糊查询，不传默认查询全部
//     * @param bean
//     * @return
//     */
//    List<MapBean> getHospitalSelect(MapBean bean);
//
//    List<MapBean> getPackageType();
//
//    List<MapBean> getPersonType();
//
//    List<MapBean> getSpecialPost();
//
//    List<MapBean> getRole();
//
//    /**
//     * 获取套餐下拉框 可根据传入mapBean keyword和type值进行查询，不传默认查询全部
//     * @param bean
//     * @return
//     */
//    List<MapBean> getSetMeal(MapBean bean);
//
//    int findteltpone(String phone);
//
//    String getstandId();
//
//    int userteltpone(String phone);
//
//    List<MapBean> getHazard();


}
